﻿//////////////////////////////////////////////
// Node.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkExport
#include "../Dll/DllDefines.h"

// nkMemory
#include <NilkinsMemory/Containers/BufferCast.h>
#include <NilkinsMemory/Containers/StringView.h>

#include <NilkinsMemory/Pointers/UniquePtr.h>

// Standards
#include <string>
#include <unordered_map>
#include <utility>
#include <vector>

/// Internal classes -------------------------

namespace nkExport
{
	union NodeValueData
	{
		int intValue ;
		unsigned int uintValue ;
		long long longValue ;
		unsigned long long ulongValue ;

		float floatValue ;
		double doubleValue ;

		bool boolValue ;
	} ;

	enum NODE_VALUE_TYPE
	{
		NOT_VALUE_TYPE,
		BOOL_TYPE,
		STRING_TYPE,
		INT_TYPE,
		UINT_TYPE,
		LONG_TYPE,
		ULONG_TYPE,
		FLOAT_TYPE,
		DOUBLE_TYPE
	} ;

	enum NODE_NATURE
	{
		EMPTY_NATURE = 0,
		VALUE_NATURE,
		OBJECT_NATURE,
		ARRAY_NATURE
	} ;
}

/// Class ------------------------------------
	
namespace nkExport
{
	class DLL_EXPORT_EXPORT Node final
	{
		private :
		
			// Attributs
			// On peut avoir des valeurs
			std::pair<NodeValueData, NODE_VALUE_TYPE> _value ;
			// Cas particulier pour own la mémoire
			std::string _valueStr ;
			// Si on est un object
			std::unordered_map<std::string, nkMemory::UniquePtr<Node>> _object ;
			// Ou un array
			nkMemory::BufferCast<nkMemory::UniquePtr<Node>> _array ;

			// Ce qu'on a dans le node actuellement
			NODE_NATURE _nodeNature ;

			// Données user possibles
			void* _userData  ;
			
		public :

			// Constructeur, destructeur
			Node () ;
			Node (const Node& other) ;
			Node (Node&& other) ;
			~Node () ;

			// Getters
			// Globals
			NODE_NATURE getNodeNature () const ;
			NODE_VALUE_TYPE getNodeValueType () const ;
			// Utils
			bool isValueTypeNumber () const ;
			bool hasChild (nkMemory::StringView key, NODE_VALUE_TYPE valueType) ;
			bool hasChild (nkMemory::StringView key, NODE_NATURE nature) ;
			// Value
			nkMemory::StringView getValueAsString () const ;
			int getValueAsInt () const ;
			unsigned int getValueAsUint () const ;
			long long getValueAsLong () const ;
			unsigned long long getValueAsUlong () const ;
			float getValueAsFloat () const ;
			double getValueAsDouble () const ;
			bool getValueAsBool () const ;
			// Object
			unsigned int getMemberCount () const ;
			nkMemory::StringView getMemberKey (unsigned int index) ;
			Node* getMemberValue (unsigned int index) ;
			Node* getMemberValue (nkMemory::StringView key) ;
			Node* getMemberValue (nkMemory::StringView key, NODE_VALUE_TYPE valueType) ;
			Node* getMemberValue (nkMemory::StringView key, NODE_NATURE nodeNature) ;
			// Array
			unsigned int getArraySize () const ;
			Node* getArrayElement (unsigned int index) ;
			Node* getArrayElement (unsigned int index, NODE_VALUE_TYPE valueType) ;
			Node* getArrayElement (unsigned int index, NODE_NATURE nodeNature) ;
			// User
			void* getUserData () const ;

			// Setters
			// Value
			void setAsString (const nkMemory::StringView& value) ;
			void setAsInt (int value) ;
			void setAsUint (unsigned int value) ;
			void setAsLong (long long value) ;
			void setAsUlong (unsigned long long value) ;
			void setAsFloat (float value) ;
			void setAsDouble (double value) ;
			void setAsBool (bool value) ;
			// Object
			Node* addMember (nkMemory::StringView key) ;
			void deleteMember (nkMemory::StringView key) ;
			// Array
			Node* addElement () ;
			// User data
			void setUserData (void* value) ;

			// Operators
			Node& operator= (const Node& other) ;
			Node& operator= (Node&& other) ;
	} ;
}