﻿//////////////////////////////////////////////
// ShaderManager.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkGraphics
{
	class Shader ;
}

/// Includes ---------------------------------

// nkGraphics
#include "../Dll/DllDefines.h"

// nkCommon
#include <nilkinsCommon/Patterns/SingletonClass.h>

// nkMemory
#include <NilkinsMemory/Containers/StringView.h>

// Standards
#include <unordered_map>
#include <mutex>
#include <string>
	
/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT ShaderManager final : public nkCommon::SingletonClass<ShaderManager>
	{
		friend class nkCommon::SingletonClass<ShaderManager> ;

		private :
		
			// Attributs
			// Mutex, WIP
			std::mutex _mutex ;

			// Shader memory
			std::unordered_map<std::string, Shader*> _shaderMap ;

			// Id counter
			unsigned int _shaderCounter ;

		private :

			// Functions
			// Constructor
			ShaderManager () ;

		public :

			// Available default shaders names
			inline static const nkMemory::StringView DEFAULT_EMPTY_SHADER = "NILKINS_EMPTY_SHADER" ;
			inline static const nkMemory::StringView DEFAULT_POSITION_SHADER = "NILKINS_POSITION_SHADER" ;
			inline static const nkMemory::StringView DEFAULT_UV_SHADER = "NILKINS_UV_SHADER" ;
			inline static const nkMemory::StringView DEFAULT_NORMAL_SHADER = "NILKINS_NORMAL_SHADER" ;
			inline static const nkMemory::StringView DEFAULT_UV_NORMAL_SHADER = "NILKINS_UV_NORMAL_SHADER" ;
		
		public :

			// Destructor
			~ShaderManager () ;

			// Shader manipulations
			Shader* createOrRetrieve (const nkMemory::StringView& name) ;
			Shader* get (const nkMemory::StringView& name) ;
			Shader* getByIndex (unsigned int index) ;
			void rename (const nkMemory::StringView& oldName, const nkMemory::StringView& newName) ;
			void erase (const nkMemory::StringView& name) ;
	} ;
}