﻿//////////////////////////////////////////////
// JpgDecompressionUtils.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Includes ---------------------------------

// nkImages
#include "../Data/JpgHuffmanTable.h"
#include "../Data/JpgQuantizationTable.h"

#include "JpgBitStream.h"

/// Class ------------------------------------

namespace nkImages::jpg
{
	class JpgDecompressionUtils
	{
		public :

			// Arithmetic
			static float clamp255 (float x) ;

			// Dct
			static void inverseDct8x8 (float* in, float* out) ;

			// Jpeg structure
			static bool isMarker (const unsigned char* ptr, unsigned char suffix) ;
			static const unsigned char* getToNextMarker (const unsigned char* data) ;

			// Huffman
			static std::vector<unsigned int> buildHuffmanLut (const JpgHuffmanTable& table) ;
			static unsigned char findNextHuffmanSymbol (JpgBitStream& dataStream, const std::vector<unsigned int>& huffmanLut, unsigned int maxLength) ;

			// Mcu logic
			static int getDcFromCode (unsigned int code, unsigned int length) ;
			static void unpackAcRle (float* dctMcu, JpgBitStream& dataStream, const std::vector<unsigned int>& huffmanLut, unsigned int maxLength, const JpgQuantizationTable& quantizationTable) ;
			static void decodeNextMcu (JpgBitStream& dataStream, float* dctMcu, float* currentMcu, float& lastDcValue, const std::vector<unsigned int>& huffmanLutDc, const std::vector<unsigned int>& huffmanLutAc, const JpgHuffmanTable& huffmanTableDc, const JpgHuffmanTable& huffmanTableAc, const JpgQuantizationTable& quantizationTable) ;
	} ;
}